@@ -22,9 +22,7 @@ import android.util.Log;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 import java.util.concurrent.CopyOnWriteArrayList;
 
 @RobocopTarget
@@ -44,12 +42,12 @@ public final class EventDispatcher extends JNIObject {
     private static final int DEFAULT_BACKGROUND_EVENTS_COUNT = 64; // Empirically measured
 
     // GeckoBundle-based events.
-    private final Map<String, List<BundleEventListener>> mGeckoThreadListeners =
-        new HashMap<String, List<BundleEventListener>>(DEFAULT_GECKO_EVENTS_COUNT);
-    private final Map<String, List<BundleEventListener>> mUiThreadListeners =
-        new HashMap<String, List<BundleEventListener>>(DEFAULT_UI_EVENTS_COUNT);
-    private final Map<String, List<BundleEventListener>> mBackgroundThreadListeners =
-        new HashMap<String, List<BundleEventListener>>(DEFAULT_BACKGROUND_EVENTS_COUNT);
+    private final MultiMap<String, BundleEventListener> mGeckoThreadListeners =
+        new MultiMap<>(DEFAULT_GECKO_EVENTS_COUNT);
+    private final MultiMap<String, BundleEventListener> mUiThreadListeners =
+        new MultiMap<>(DEFAULT_UI_EVENTS_COUNT);
+    private final MultiMap<String, BundleEventListener> mBackgroundThreadListeners =
+        new MultiMap<>(DEFAULT_BACKGROUND_EVENTS_COUNT);
 
     private boolean mAttachedToGecko;
     private final NativeQueue mNativeQueue;
@@ -104,7 +102,7 @@ public final class EventDispatcher extends JNIObject {
     }
 
     private <T> void registerListener(final Class<?> listType,
-                                      final Map<String, List<T>> listenersMap,
+                                      final MultiMap<String, T> listenersMap,
                                       final T listener,
                                       final String[] events) {
         try {
@@ -113,18 +111,10 @@ public final class EventDispatcher extends JNIObject {
                     if (event == null) {
                         continue;
                     }
-                    List<T> listeners = listenersMap.get(event);
-                    if (listeners == null) {
-                        // Java doesn't let us put Class<? extends List<T>> as the type for listType.
-                        @SuppressWarnings("unchecked")
-                        final Class<? extends List<T>> type = (Class) listType;
-                        listeners = type.newInstance();
-                        listenersMap.put(event, listeners);
-                    }
-                    if (!BuildConfig.RELEASE_OR_BETA && listeners.contains(listener)) {
+                    if (!BuildConfig.RELEASE_OR_BETA && listenersMap.containsEntry(event, listener)) {
                         throw new IllegalStateException("Already registered " + event);
                     }
-                    listeners.add(listener);
+                    listenersMap.add(event, listener);
                 }
             }
         } catch (final Exception e) {
@@ -132,14 +122,14 @@ public final class EventDispatcher extends JNIObject {
         }
     }
 
-    private void checkNotRegisteredElsewhere(final Map<String, ?> allowedMap,
+    private void checkNotRegisteredElsewhere(final MultiMap<String, ?> allowedMap,
                                              final String[] events) {
         if (BuildConfig.RELEASE_OR_BETA) {
             // for performance reasons, we only check for
             // already-registered listeners in non-release builds.
             return;
         }
-        for (final Map<String, ?> listenersMap : Arrays.asList(mGeckoThreadListeners,
+        for (final MultiMap<String, ?> listenersMap : Arrays.asList(mGeckoThreadListeners,
                                                                mUiThreadListeners,
                                                                mBackgroundThreadListeners)) {
             if (listenersMap == allowedMap) {
@@ -147,7 +137,7 @@ public final class EventDispatcher extends JNIObject {
             }
             synchronized (listenersMap) {
                 for (final String event : events) {
-                    if (listenersMap.get(event) != null) {
+                    if (listenersMap.containsKey(event)) {
                         throw new IllegalStateException(
                             "Already registered " + event + " under a different type");
                     }
@@ -156,7 +146,7 @@ public final class EventDispatcher extends JNIObject {
         }
     }
 
-    private <T> void unregisterListener(final Map<String, List<T>> listenersMap,
+    private <T> void unregisterListener(final MultiMap<String, T> listenersMap,
                                         final T listener,
                                         final String[] events) {
         synchronized (listenersMap) {
@@ -164,9 +154,8 @@ public final class EventDispatcher extends JNIObject {
                 if (event == null) {
                     continue;
                 }
-                List<T> listeners = listenersMap.get(event);
-                if ((listeners == null ||
-                     !listeners.remove(listener)) && !BuildConfig.RELEASE_OR_BETA) {
+
+                if (!listenersMap.remove(event, listener) && !BuildConfig.RELEASE_OR_BETA) {
                     throw new IllegalArgumentException(event + " was not registered");
                 }
             }
@@ -272,7 +261,7 @@ public final class EventDispatcher extends JNIObject {
         synchronized (mGeckoThreadListeners) {
             geckoListeners = mGeckoThreadListeners.get(type);
         }
-        if (geckoListeners != null && !geckoListeners.isEmpty()) {
+        if (!geckoListeners.isEmpty()) {
             final boolean onGeckoThread = ThreadUtils.isOnGeckoThread();
             final EventCallback wrappedCallback = JavaCallbackDelegate.wrap(callback);
 
@@ -318,17 +307,22 @@ public final class EventDispatcher extends JNIObject {
             return true;
         }
 
-        Log.w(LOGTAG, "No listener for " + type);
+        final String error = "No listener for " + type;
+        if (callback != null) {
+            callback.sendError(error);
+        }
+
+        Log.w(LOGTAG, error);
         return false;
     }
 
     @WrapForJNI
     public boolean hasListener(final String event) {
-        for (final Map<String, ?> listenersMap : Arrays.asList(mGeckoThreadListeners,
+        for (final MultiMap<String, ?> listenersMap : Arrays.asList(mGeckoThreadListeners,
                 mUiThreadListeners,
                 mBackgroundThreadListeners)) {
             synchronized (listenersMap) {
-                if (listenersMap.get(event) != null) {
+                if (listenersMap.containsKey(event)) {
                     return true;
                 }
             }
@@ -340,20 +334,14 @@ public final class EventDispatcher extends JNIObject {
     private boolean dispatchToThread(final String type,
                                      final GeckoBundle message,
                                      final EventCallback callback,
-                                     final Map<String, List<BundleEventListener>> listenersMap,
+                                     final MultiMap<String, BundleEventListener> listenersMap,
                                      final Handler thread) {
         // We need to hold the lock throughout dispatching, to ensure the listeners list
         // is consistent, while we iterate over it. We don't have to worry about listeners
         // running for a long time while we have the lock, because the listeners will run
         // on a separate thread.
         synchronized (listenersMap) {
-            final List<BundleEventListener> listeners = listenersMap.get(type);
-            if (listeners == null) {
-                return false;
-            }
-
-            if (listeners.isEmpty()) {
-                // There were native listeners, and they're gone.
+            if (!listenersMap.containsKey(type)) {
                 return false;
             }
 
@@ -361,7 +349,7 @@ public final class EventDispatcher extends JNIObject {
             final EventCallback wrappedCallback = JavaCallbackDelegate.wrap(callback);
 
             // Event listeners will call | callback.sendError | if applicable.
-            for (final BundleEventListener listener : listeners) {
+            for (final BundleEventListener listener : listenersMap.get(type)) {
                 thread.post(new Runnable() {
                     @Override
                     public void run() {
