@@ -41,6 +41,7 @@
 #include "nsIOService.h"
 #include "nsIPrompt.h"
 #include "nsIProperties.h"
+#include "nsISerialEventTarget.h"
 #include "nsITokenPasswordDialogs.h"
 #include "nsIWindowWatcher.h"
 #include "nsIXULRuntime.h"
@@ -2090,6 +2091,26 @@ nsresult nsNSSComponent::InitializeNSS() {
   }
 }
 
+NS_IMETHODIMP
+nsNSSComponent::DispatchTaskToSerialBackgroundQueue(nsIRunnable* task) {
+  if (!NS_IsMainThread()) {
+    nsCOMPtr<nsIRunnable> mainThreadRunnable(NS_NewRunnableFunction(
+        "DispatchTaskToSerialBackgroundQueue::mainThreadRunnable",
+        [self = RefPtr<nsNSSComponent>(this),
+         taskHandle = nsCOMPtr<nsIRunnable>(task)]() -> void {
+          nsresult rv = self->DispatchTaskToSerialBackgroundQueue(taskHandle);
+          Unused << NS_WARN_IF(NS_FAILED(rv));
+        }));
+    return NS_DispatchToMainThread(mainThreadRunnable.forget());
+  }
+
+  if (mBackgroundTaskQueue) {
+    return mBackgroundTaskQueue->Dispatch(task, NS_DISPATCH_EVENT_MAY_BLOCK);
+  }
+
+  return NS_ERROR_NOT_AVAILABLE;
+}
+
 void nsNSSComponent::ShutdownNSS() {
   MOZ_LOG(gPIPNSSLog, LogLevel::Debug, ("nsNSSComponent::ShutdownNSS\n"));
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
@@ -2115,6 +2136,8 @@ void nsNSSComponent::ShutdownNSS() {
 
   Preferences::RemoveObserver(this, "security.");
 
+  mBackgroundTaskQueue = nullptr;
+
   // Release the default CertVerifier. This will cause any held NSS resources
   // to be released.
   MutexAutoLock lock(mMutex);
@@ -2147,6 +2170,14 @@ nsresult nsNSSComponent::Init() {
     return rv;
   }
 
+  rv = NS_CreateBackgroundTaskQueue("nsNSSComponent::mBackgroundTaskQueue",
+                                    getter_AddRefs(mBackgroundTaskQueue));
+  if (NS_FAILED(rv)) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Error,
+            ("NS_CreateBackgroundTaskQueue failed"));
+    return rv;
+  }
+
   return RegisterObservers();
 }
 
